/* Copyright (c) 2003, Niel Markwick

 * All rights reserved.

 * 

 * Redistribution and use in source and binary forms, with or without

 * modification, are permitted provided that the following conditions are

 * met:

 * 

 * Redistributions of source code must retain the above copyright notice,

 * this list of conditions and the following disclaimer.

 * 

 * Redistributions in binary form must reproduce the above copyright

 * notice, this list of conditions and the following disclaimer in the

 * documentation and/or other materials provided with the distribution.

 * 

 * Neither the name of this project  nor the names of its

 * contributors may be used to endorse or promote products derived from

 * this software without specific prior written permission.

 * 

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS

 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT

 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR

 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT

 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,

 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT

 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,

 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY

 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT

 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE

 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 */



#include <windows.h>

#include "ammo.h"				// Required for ATI Remote Wonder plug-in SDK API

#include "resource.h"

#include <string.h>

#include <stdio.h>





// ddl instance

static HINSTANCE g_hInstance = NULL;





// Classnames of app ans window

#define DEFSAGETVAPPCLASS "SageApp"

#define DEFSAGECLIENTAPPCLASS "SageClientApp"

#define DEFSAGETVWINCLASS "sage.SpecialWindow"

static char SageTvAppClass[80]={0};

static char SageClientAppClass[80]={0};

static char SageTvWinClass[80]={0};



// Structure containing SendMessage codes read from ini file

// 

struct code_def_s { 

	char *ininame;

	char *menuname;

	int message;

	int param1;

	int param2;

};



// The functions assigned to the programmable keys from the ini file.

static struct code_def_s *functionCodes = NULL;

static int kNumFunctions =0; 



// Default set of fucntion codes 

// -- must be in format returned by GetPrivateProfileSection 

// -- ie each line terminated by \0, end of data indicated by \0\0



#define DEF_FUNCCODES									\

        "ViewTV=Sage|View TV,1258,0,15\0"                       \

        "ViewGuide=Sage|View Guide,1258,0,18\0"                 \

        "ViewLibrary=Sage|Library Menu,1258,1,42\0"             \

        "ViewJukebox=Sage|Music Jukebox,1258,1,84\0"            \

        "ViewSchedule=Sage|Recording Schedule,1258,1,85\0"      \

        "ViewRecordings=Sage|Sage Recordings,1258,1,86\0"       \

        "Search=Sage|Search Menu,1258,1,40\0"                   \

        "Setup=Sage|Setup Menu,1258,1,41\0"                     \

        "LiveTV=Sage|View Live TV,1258,1,63\0"                  \

        "Time_Scroll=Video|Time Scroll,1258,1,10\0"             \

        "Faster=Video|Faster,1258,1,16\0"                       \

        "Slower=Video|Slower,1258,1,17\0"                       \

        "FF2=Video|Skip Forward #2,1258,1,61\0"                 \

        "REW2=Video|Skip Backward #2,1258,1,62\0"               \

        "Watched=Guide|Watched,1258,1,21\0"                     \

        "Favourite=Guide|Favourite,1258,1,22\0"                 \

        "Dont_Like=Guide|Dont Like,1258,1,23\0"                 \

        "Info=Misc|Info,1258,1,24\0"                            \

        "Power=Misc|Sleep,1258,1,19\0"                          \

        "Options=Misc|Options,1258,1,29\0"                      \

        "AspectRatioFill=Aspect Ratio|Fill,1258,1,47\0"         \

        "AspectRatio43=Aspect Ratio|4x3,1258,1,48\0"            \

        "AspectRatio169=Aspect Ratio|16x9,1258,1,49\0"          \

        "AspectRatioSource=Aspect Ratio|Source,1258,1,50\0"     \

        "Home=Sage|Home Screen,1258,1,28\0"                     \

        "Back=Sage|Prev Screen,1258,1,75\0"                     \

        "Forward=Sage|Next Screen,1258,1,76\0"                  \

        "Delete=Guide|Delete,1258,1,83\0"                       \

        "ViewPicLib=Sage|Picture Library,1258,1,87\0"           \

        "ViewVidLib=Sage|Video Library,1258,1,88\0"             \

	"Stop=Video|Stop Playback/Close File,1258,1,89\0"       \

        "\0\0"





// Structure to define the Sendmessage codes for the non-programmable keys

// #define constants are used to lookup in this array, so must be correct.

static struct code_def_s KeyCodes[] = 

{

#define kKeyChannelUP (0)

    {

        "ChannelUp",

        NULL,

        1258,1,11,

    },

#define kKeyChannelDown (1)

    {

        "ChannelDown",

        NULL,

        1258,1,12,

    },

#define kKeyVolumeUp (2)

    {

        "VolumeUp",

        NULL,

        1258,1,13,

    },

#define kKeyVolumeDown (3)

    {

        "VolumeDown",

        NULL,

        1258,1,14,

    },

#define kKeyMute (4)

    {

        "Mute",

        NULL,

        1258,1,26,

    },

#define kKeyPlay (5)

    {

        "Play",

        NULL,

        1258,1,7,

    },

#define kKeyPause (6)

    {

        "Pause",

        NULL,

        1258,1,6,

    },

#define kKeyFf (7)

    {

        "FF",

        NULL,

        1258,1,8,

    },

#define kKeyRew (8)

    {

        "REW",

        NULL,

        1258,1,9,

    },

#define kKeyRecord (9)

    {

        "Record",

        NULL,

        1258,1,25,

    },

#define kKeyStop (10)

    {

        "Stop", //Home

        NULL,

        1258,1,28,

    },

#define kKeyMenu (11)

    {

        "Menu", //  Menu key (bottom left of number pad) == info

        NULL,

        1258,1,24,

    },

#define kKeySetup (12)

    {

        "Setup", // 'tick' key (bottom right of number pad) == Options

        NULL,

        1258,1,29,

    },

#define kKeyStopwatch (13)

    {

        "Stopwatch", // Stopwatch (left of cursor block) -> Time Scroll

        NULL,

        1258,1,10,

    },

#define kKeyResize (14)

    {

        "Resize", // Resize  (right of cursor block) -> full screen.

        NULL,

        1258,1,27,

    },

#define kKeyWeb (15)

    {

        "Web", // no action by default -> launch browser

        NULL,

        0,0,0,

    },

};



// The number of mapped keys  provided by the plug-in

#define kNumKeyCodes				(sizeof(KeyCodes)/sizeof(KeyCodes[0]))



void ReadIniCode(char *file,char *section, struct code_def_s* codedef)

{

	char ReadBuff[80];

	GetPrivateProfileString(section,codedef->ininame,"default",ReadBuff,sizeof(ReadBuff),file);

	if (strcmp(ReadBuff,"default") != 0 )

	{

		// read some data, decode it

		// should be int,int,int

		unsigned int vals[3] = {0};

		sscanf(ReadBuff," %u , %u , %u ",&vals[0],&vals[1],&vals[2]);

		codedef->message=vals[0];

		codedef->param1=vals[1];

		codedef->param2=vals[2];

	}



}



void ReadIniFunctions(char *inifile)

{

    char Buffer[4096] = {0};

    char *p=Buffer;

    int num_entries=0;

    if ( GetPrivateProfileSection("SageTVFunctions",Buffer,sizeof(Buffer),inifile) < 5 )

    {

		// less than 5 chars returned? defaults!

		memcpy(Buffer,DEF_FUNCCODES,sizeof(DEF_FUNCCODES));

    }

    

    // count number of entries -- relies on last string having two \0 characters

    while(*p != '\0')

    {

		// find end of string

		// eg p->"helloworld"

		// strlen(p)=10

		// p+10 -> trailing \0

		// p+10+1 -> char after trailing \0

		p+=strlen(p)+1;

		num_entries++;

    }

    

    functionCodes=malloc(sizeof(functionCodes[0])*num_entries);

    memset(functionCodes,0,sizeof(functionCodes[0])*num_entries);



    p=Buffer;

    num_entries=0;

    while(*p!='\0')

    {

		// process each entry

		// entry should be in format:

		// Name=MenuName,msg,param1,param2

		

		unsigned int vals[3] = {0};

		

		// find '='

		char *p_equals=strstr(p,"=");

		if ( p_equals )

		{

			char *p_name=NULL;

			*p_equals='\0';

			

			//ininame=Buffer,

			// find next non-space



			p=p_equals+1;

			while (*p != '\0' && ( *p==' ' || *p == '\t') )

				p++;

			p_name=p;



			// menuname=p_name;

			// find next ','

			while (*p != '\0' && *p!=',') 

				p++;

			// end memuname

			*p='\0';

			p++;



			// get values

			sscanf(p," %u , %u , %u ",&vals[0],&vals[1],&vals[2]);

			if ( Buffer[0] && p_name[0] && vals[0]  )

			{

			// seems reasonable!

				functionCodes[num_entries].ininame = strdup(Buffer);

				functionCodes[num_entries].menuname = strdup(p_name);

				functionCodes[num_entries].message = vals[0];

				functionCodes[num_entries].param1 = vals[1];

				functionCodes[num_entries].param2 = vals[2];

				num_entries++;

			}

			

		}

		p+=strlen(p)+1;

    }

    kNumFunctions=num_entries;

}



BOOL APIENTRY  DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved ) 

{ 

	HKEY hKey =0;

#define BUFSIZE       1024

	char szRemDir[BUFSIZE] = {0};

	DWORD dwBuf=BUFSIZE;

	int i;



    if( dwReason==DLL_PROCESS_ATTACH ) { 

        g_hInstance = hInstance; 

    } 



	if ( ERROR_SUCCESS == RegOpenKeyEx( HKEY_LOCAL_MACHINE,"SOFTWARE\\ATI Technologies\\Multimedia\\Remote Wonder", 0, KEY_QUERY_VALUE, &hKey ) )

	{

		if ( ERROR_SUCCESS == RegQueryValueEx( hKey, "PluginsPath", NULL, NULL, (LPBYTE) szRemDir, &dwBuf) )

		{ 

			RegCloseKey( hKey );

		}

	}

	// if registry could not be opened, filename will be "\SageTV.ini"

	// This means that the defautl values will be applied...

	strcat(szRemDir,"\\SageTV.ini");	



	ReadIniFunctions(szRemDir);

			

	for ( i = 0 ; i < kNumKeyCodes ; i ++ )

		ReadIniCode(szRemDir,"SageTVKeys",	&KeyCodes[i]);



	GetPrivateProfileString("Config","SageTvAppClass",DEFSAGETVAPPCLASS,SageTvAppClass,sizeof(SageTvAppClass),szRemDir);

	GetPrivateProfileString("Config","SageClientAppClass",DEFSAGECLIENTAPPCLASS,SageClientAppClass,sizeof(SageClientAppClass),szRemDir);

	GetPrivateProfileString("Config","SageTvWinClass",DEFSAGETVWINCLASS,SageTvWinClass,sizeof(SageTvWinClass),szRemDir);

			

	

	return TRUE;     

} 













// Function prototype to a function that handles programmed key events.

static BOOL HandleProgrammedKey (HWND hCurrWin, WORD wKeyEvent, WORD wState);



/*

 * Function:	WhatKeysDoYouWant

 *

 * Purpose:		Determines what groups of keys should be re-directed to

 *				the plug-in.

 *

 * Inputs:		None

 *

 * Returns:		A bit mask which indicates the groups of keys that should

 *				be re-directed to this plug-in.

 *

 * When called: This function is called once when the plug-in is first loaded,

 *				and again after calling Configure().

 *

 * Author:		ATI

 *

 * Copyright:	Copyright (c) 2002 ATI Technologies Inc

 */

DWORD WhatKeysDoYouWant (void)

{

	// For SageTV we need to control the following keys:

	//

	// CUSTOM_MAPPED - provide list of programmable options

	// VOLUME_GROUP - adjust the playback volume

	// PLAY_GROUP - play, pause, stop, FF, RW

	// SETUP - toggle the preferences dialog

	// MENU - toggle the playlist editor

	// RESIZE - toggle doublesize mode

	// STOPWATCH - add current track as a bookmark



	return (CUSTOM_MAPPED

			|CHANNEL_GROUP

			|VOLUME_GROUP

			|PLAY_GROUP

			|MENU

			|SETUP

			|RECORD

			|STOPWATCH

			|RESIZE

			|(KeyCodes[kKeyWeb].message!=0?WEB_LAUNCH:0));

}





/*

 * Function:	EnumerateProgrammableFunction

 *

 * Purpose:		Enumerate the descriptions of the programamble

 *				functions provided by this plug-in.

 *

 * Inputs:		wIndex - index of requested description

 *

 * Returns:		Pointer to the programmable function description

 *				or NULL if wIndex is greater than the number of

 *				programmable functions provided by this plug-in.

 *

 * When called:	This function is called after any call to WhatKeysDoYouWant()

 *				that returns a request for custom mapped keys (CUSTOM_MAPPED).

 *

 * Author:		ATI

 *

 * Copyright:	Copyright (c) 2002 ATI Technologies Inc

 */

char *EnumerateProgrammableFunction (WORD wIndex)

{	

	if (wIndex >= kNumFunctions) 

		return NULL;



	// Return a pointer to the requested description (see table above)

	return functionCodes[wIndex].menuname;

}



/* Function: ConfigureDlgProc 

 * 

 * Purpose: Handles the Configure dialog box commands 

 * 

 * Inputs: hDlg - handle of configure dialog 

 * Message - message to handle 

 * wParam - first message parameter 

 * lParam - second message parameter 

 * 

 * Returns: TRUE if the message was handled, FALSE otherwise. 

 */ 

BOOL CALLBACK ConfigureDlgProc (HWND hDlg, UINT Message, WPARAM wParam, LPARAM lParam) 

{ 

	char CodesText[4096] = {0};

	int i;

	switch (Message) 

	{

	case WM_INITDIALOG:

		strcat(CodesText,

			   "The following are the Window class names used as the Targets of the SendMessage function.\r\n");

		strcat(CodesText,

			   "    SageTV Application Window class: ");

		strcat(CodesText,SageTvAppClass);

		strcat(CodesText,

			   "\r\n"

				"    SageClient Application Window class: ");

		strcat(CodesText,SageClientAppClass);

		strcat(CodesText,

			   "\r\n"

			   "The following is the Window class name used used to determine if SageTV/Client currently has the focus.\r\n"

				"    SageTV/SageClient Main Window class: ");

		strcat(CodesText,SageTvWinClass);

			  

		strcat(CodesText,

			   "\r\n\r\n"

			   "The following are the parameters sent to the SendMessage function when the defined key is pressed. The codes are taken from the Girder GML file for SageTV (www.girder.nl), and are customisable using the sagetv.ini file in the ATI plugins directory.\r\n\r\n");

		strcat(CodesText,"Programmable button [A-F] codes:\r\n----------------------------------------\r\n");

		for ( i = 0 ; i < kNumFunctions ; i ++ )

		{

			char tmpbuff[80];

			sprintf(tmpbuff,"    %s = %d,%d,%d\r\n",

					functionCodes[i].menuname,

					functionCodes[i].message,

					functionCodes[i].param1,

					functionCodes[i].param2);

			strcat(CodesText,tmpbuff);

		}



		strcat(CodesText,"\r\nRemote Key codes:\r\n----------------------------------------\r\n");

		for ( i = 0 ; i < kNumKeyCodes ; i ++ )

		{

			char tmpbuff[80];

			if ( KeyCodes[i].message )

			{

				sprintf(tmpbuff,"    %s = %d,%d,%d\r\n",

						KeyCodes[i].ininame,

						KeyCodes[i].message,

						KeyCodes[i].param1,

						KeyCodes[i].param2);

				strcat(CodesText,tmpbuff);

			}

			else

			{

				sprintf(tmpbuff,"    %s <default action>\r\n",

						KeyCodes[i].ininame);

				strcat(CodesText,tmpbuff);

			}

		}



		SetDlgItemText(hDlg,IDC_ABOUT_EDIT,CodesText); 

		break;





	case WM_COMMAND: 

		// Close the dialog if OK or CANCEL selected 

		if (LOWORD(wParam) == IDOK) 

		{	 

			EndDialog(hDlg, LOWORD(wParam));

			return TRUE; 

		} 

		if (LOWORD(wParam) == IDCANCEL) 

		{ 

			EndDialog(hDlg, LOWORD(wParam)); 

			return TRUE; 

		} 

	}; 

	return FALSE; 

} 



/* 

 * Function: Configure 

 * 

 * Purpose: Displays any configuration/setup page required by 

 * this plug-in. 

 * 

 * Inputs: hWnd - handle to the parent window. The parent window 

 * is used to ensure correct modal behaviour when creating 

 * dialog and message boxes in this plug-in. 

 * 

 * Returns: 	None

 *

 * When called: This function is called when the "Configure" button in the

 *				ATI Remote Wonder software is pressed.

 *

 * Author:		ATI

 *

 * Copyright:	Copyright (c) 2002 ATI Technologies Inc

 */ 

void    Configure (HANDLE hWnd) 

{ 

	FARPROC dlgConfigure; 



	dlgConfigure = MakeProcInstance((FARPROC) ConfigureDlgProc, g_hInstance);

	DialogBox(g_hInstance, MAKEINTRESOURCE(IDD_ABOUT), (HWND) hWnd, (DLGPROC) dlgConfigure);

	FreeProcInstance(dlgConfigure);

} 







/*

 * Function:	AreYouInFocus

 *

 * Purpose:		Determines if this plug-in is currently in-focus.

 *

 * Inputs:		None

 *

 * Returns:		TRUE if the plug-in is currently in focus, FALSE

 *				otherwise.

 *

 * When called:	This function is called when a key event occurs that

 *				the plug-in has requested.

 *

 * Author:		ATI

 *

 * Copyright:	Copyright (c) 2002 ATI Technologies Inc

 */

int AreYouInFocus (void)

{

	

	HWND hTestWin= GetForegroundWindow();

	char ClassName[256]="Unknown";





	// iterate up tree from foreground window attempting to match

	// the window class names with the predefined class names

	while ( hTestWin )

	{

		GetClassName(hTestWin,ClassName,sizeof(ClassName));

		if ( stricmp(ClassName,SageTvWinClass) == 0 )

			return TRUE;

		if ( stricmp(ClassName,SageTvAppClass) == 0 )

			return TRUE;

		if ( stricmp(ClassName,SageClientAppClass) == 0 )

			return TRUE;

		hTestWin = GetWindow(hTestWin, GW_OWNER);

	}



	return FALSE;

}





/*

 * Function:	HandleKey

 *

 * Purpose:		Handles remote control keys.

 *

 * Inputs:		bCustom   - TRUE if wKeyEvent has been mapped to a custom

 *						    programamble option.

 *				wKeyEvent - Remote control key code for button that was

 *							pressed.

 *				wState    - RMCTRL_KEY_ON, when button is first pressed

 *						    RMCTRL_KEY_REPEAT, when button is held down

 *						    RMCTRL_KEY_OFF, when button is released

 *

 * Returns:		TRUE if the plug-in handled the command, FALSE otherwise.

 *

 * When called: This function is called whenever a key event occurs that

 *				the plug-in has requested when the plug-in is in focus.

 *

 * Author:		ATI

 *

 * Copyright:	Copyright (c) 2002 ATI Technologies Inc

 */

BOOL HandleKey (BOOL bCustom, WORD wKeyEvent, WORD wState)

{

	int trans_key=-1;
	
	int repeat_key_counter=0;	

	// Get the APPL WINDOW to send the PostMessage data to,,,



	// Considering that there can be SageTV or multiple SageClient windows in focus, this gets complicated:

	//

	// we do it by... 

	//	Getting foreground window

	//  Getting PID of foreground window

	//  Enumerating child windows of desktop with class SageApp

	//     -> checking each for match of PID

	//  Enumerating child windows of desktop with class SageClientApp

	//     -> checking each for match of PID

	//  



	HWND hCurrWin = GetForegroundWindow();

	HWND hTargetWin = NULL;

	DWORD dwCurrPid = 0;

	DWORD dwCurrTid = GetWindowThreadProcessId(hCurrWin,&dwCurrPid);





	// Look for SageApp windows

	HWND hTestWin=FindWindowEx(NULL,NULL,SageTvAppClass,NULL);

	while ( hTargetWin == NULL &&

			hTestWin != NULL)

	{

		DWORD dwTestPid=0;

		DWORD dwTestTid=GetWindowThreadProcessId(hTestWin,&dwTestPid);

		if ( dwTestPid==dwCurrPid)

		{	

			hTargetWin=hTestWin;

			break;

		}

		

		/// look for next window

		hTestWin=FindWindowEx(NULL,hTestWin,SageTvAppClass,NULL);

	}



	// Look for SageClientApp windows

	hTestWin=FindWindowEx(NULL,NULL,SageClientAppClass,NULL);

	while ( hTargetWin == NULL &&

			hTestWin != NULL)

	{

		DWORD dwTestPid=0;

		DWORD dwTestTid=GetWindowThreadProcessId(hTestWin,&dwTestPid);

		if ( dwTestPid==dwCurrPid)

		{	

			hTargetWin=hTestWin;

			break;

		}

		

		// look for next window

		hTestWin=FindWindowEx(NULL,hTestWin,SageClientAppClass,NULL);

	}

	





	// If the main window is not found SageTV has been closed.

	if (hTargetWin == NULL)

	{

		// It is important to return FALSE

		// if the key was not processed so that other plug-ins or the ATI Remote Wonder

		// software can handle the key press.

		return FALSE;

	}

	

	// Handle programmed keys seperately

	if (bCustom)	

	{

		return HandleProgrammedKey(hTargetWin, wKeyEvent, wState);	

	}



	// Handle non-programmed keys

	if (wState == RMCTRL_KEY_ON)

	{



		switch (wKeyEvent)

		{



		case RMCTRL_CHANNELUP:

			trans_key=kKeyChannelUP;

			break;

		case RMCTRL_CHANNELDOWN:

			trans_key=kKeyChannelDown;

			break;

		case RMCTRL_VOLUMEUP:

			trans_key=kKeyVolumeUp;

			break;

		case RMCTRL_VOLUMEDOWN:

			trans_key=kKeyVolumeDown;

			break;

		case RMCTRL_MUTE:

			trans_key=kKeyMute;

			break;

		case RMCTRL_PLAY:

			trans_key=kKeyPlay;

			break;

		case RMCTRL_PAUSE:

			trans_key=kKeyPause;

			break;

		case RMCTRL_FF:

			trans_key=kKeyFf;

			break;

		case RMCTRL_RW:

			trans_key=kKeyRew;

			break;

		case RMCTRL_RECORD:

			trans_key=kKeyRecord;

			break;

		case RMCTRL_STOP:

			trans_key=kKeyStop;

			break;

		case RMCTRL_MENU:

			trans_key=kKeyMenu;

			break;

		case RMCTRL_SETUP:

			trans_key=kKeySetup;

			break;

		case RMCTRL_STOPWATCH:

			trans_key=kKeyStopwatch;

			break;

		case RMCTRL_RESIZE:

			trans_key=kKeyResize;

			break;

		case RMCTRL_WEB_LAUNCH:

			trans_key=kKeyWeb;

			break;

		}



	}

	else if (wState == RMCTRL_KEY_REPEAT)	

	{

		repeat_key_counter++;

		switch (wKeyEvent)

		{

		case RMCTRL_CHANNELUP:

			trans_key=kKeyChannelUP;

			break;

		case RMCTRL_CHANNELDOWN:

			trans_key=kKeyChannelDown;

			break;

		case RMCTRL_VOLUMEUP:

			trans_key=kKeyVolumeUp;

			break;

		case RMCTRL_VOLUMEDOWN:

			trans_key=kKeyVolumeDown;

			break;

		case RMCTRL_FF:

			trans_key=kKeyFf;

			break;

		case RMCTRL_RW:

			trans_key=kKeyRew;

			break;

		}			

	}



	if ( trans_key >= 0 && 

		 trans_key <kNumKeyCodes &&
		 
		 KeyCodes[trans_key].message != 0) {

		// Use PostMessage to send a command to the Sage window. For the remote

		// control to remain responsive HandleKey must return quickly. If you

		// use SendMessage here and Sage opens a dialog HandleKey will not

		// return until the dialog is closed. 

		if (repeat_key_counter%2 == 0) {
			
			PostMessage(hTargetWin, 
	
				KeyCodes[trans_key].message,
	
				KeyCodes[trans_key].param1,
	
				KeyCodes[trans_key].param2);

		}
					
		return TRUE;

	}

	// The plug-in did not process this key. It is important to return FALSE

	// if the key was not processed so that other plug-ins or the ATI Remote Wonder

	// software can handle the key press.

	return FALSE;

}





/*

 * Function:	HandleProgrammedKey

 *

 * Purpose:		Handles remote control programmed keys

 *

 * Inputs:		hTargetWin	  - Handle to the SageTV app window

 *				wKeyEvent - Remote control key code for button that was

 *							pressed.

 *				wState    - RMCTRL_KEY_ON, when button is first pressed

 *						    RMCTRL_KEY_REPEAT, when button is held down

 *						    RMCTRL_KEY_OFF, when button is released	

 *

 * Returns:		TRUE if the plug-in handled the command, FALSE otherwise.

 *

 * When called:	This function is called when a custom mapped programmable

 *				key has been pressed and the plug-in is in focus. This is

 *				a helper function for HandleKey().

 *

 *

 * Copyright:	Copyright (c) 2002 ATI Technologies Inc

 */

static BOOL HandleProgrammedKey (HWND hTargetWin, WORD wKeyEvent, WORD wState)

{

	if (wState != RMCTRL_KEY_ON)

		return FALSE;



	if ( wKeyEvent < 0 || wKeyEvent >= kNumFunctions )

		// out of range

		return FALSE;



	if ( functionCodes[wKeyEvent].message != 0 )

	{

	    PostMessage(hTargetWin, 

					functionCodes[wKeyEvent].message, 

					functionCodes[wKeyEvent].param1, 

					functionCodes[wKeyEvent].param2);

	    return TRUE;

	}

	return FALSE;

}

